@@ -61,6 +61,8 @@
 #include "nsContentUtils.h"
 #include "nsUnicharUtils.h"
 #include "nsAutoPtr.h"
+#include "nsIEventQueue.h"
+#include "nsEventQueueUtils.h"
 
 static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
 
@@ -171,6 +173,39 @@ nsScriptLoadRequest::FireScriptEvaluated(nsresult aResult)
   }
 }
 
+//////////////////////////////////////////////////////////////
+// PLEvent for delayed running of scripts
+//////////////////////////////////////////////////////////////
+
+class nsScriptLoaderEvent : public PLEvent
+{
+public:
+  nsScriptLoaderEvent(nsScriptLoader* aScriptLoader)
+    : mScriptLoader(aScriptLoader)
+  {
+    PL_InitEvent(this, aScriptLoader, Handle, Destroy);
+  }
+  
+  PR_STATIC_CALLBACK(void*) Handle(PLEvent* aEvent);
+  PR_STATIC_CALLBACK(void) Destroy(PLEvent* aEvent);
+
+  nsRefPtr<nsScriptLoader> mScriptLoader;
+};
+
+/* static */ void * PR_CALLBACK
+nsScriptLoaderEvent::Handle(PLEvent* aEvent)
+{
+  NS_STATIC_CAST(nsScriptLoaderEvent*, aEvent)->mScriptLoader->ProcessPendingReqests();
+
+  return nsnull;
+}
+
+/* static */ void PR_CALLBACK
+nsScriptLoaderEvent::Destroy(PLEvent* aEvent)
+{
+  delete NS_STATIC_CAST(nsScriptLoaderEvent*, aEvent);
+}
+
 //////////////////////////////////////////////////////////////
 //
 //////////////////////////////////////////////////////////////
@@ -490,6 +525,7 @@ nsScriptLoader::DoProcessScriptElement(nsIScriptElement *aElement,
           case '4': jsVersion = JSVERSION_1_4; break;
           case '5': jsVersion = JSVERSION_1_5; break;
           case '6': jsVersion = JSVERSION_1_6; break;
+          case '7': jsVersion = JSVERSION_1_7; break;
           default:  jsVersion = JSVERSION_UNKNOWN;
         }
       }
@@ -768,9 +804,9 @@ nsScriptLoader::EvaluateScript(nsScriptLoadRequest* aRequest,
   mCurrentScript = aRequest->mElement;
 
   PRBool isUndefined;
-  context->EvaluateString(aScript, globalObject->GetGlobalJSObject(),
-                          principal, url.get(), aRequest->mLineNo,
-                          aRequest->mJSVersion, nsnull, &isUndefined);
+  rv = context->EvaluateString(aScript, globalObject->GetGlobalJSObject(),
+                               principal, url.get(), aRequest->mLineNo,
+                               aRequest->mJSVersion, nsnull, &isUndefined);
 
   // Put the old script back in case it wants to do anything else.
   mCurrentScript = oldCurrent;
@@ -793,6 +829,28 @@ nsScriptLoader::EvaluateScript(nsScriptLoadRequest* aRequest,
   return rv;
 }
 
+void
+nsScriptLoader::ProcessPendingRequestsAsync()
+{
+  if (mPendingRequests.Count()) {
+    nsCOMPtr<nsIEventQueue> uiThreadQueue;
+    NS_GetMainEventQ(getter_AddRefs(uiThreadQueue));
+    if (!uiThreadQueue) {
+      return;
+    }
+
+    PLEvent *evt = new nsScriptLoaderEvent(this);
+    if (!evt) {
+      return;
+    }
+
+    nsresult rv = uiThreadQueue->PostEvent(evt);
+    if (NS_FAILED(rv)) {
+      PL_DestroyEvent(evt);
+    }
+  }
+}
+
 void
 nsScriptLoader::ProcessPendingReqests()
 {
