@@ -52,7 +52,7 @@
 #include "nsSVGCairoPathBuilder.h"
 #include "nsMemory.h"
 #include <float.h>
-#include <cairo.h>
+#include "cairo.h"
 #include "nsSVGCairoRegion.h"
 #include "nsISVGGradient.h"
 #include "nsSVGCairoGradient.h"
@@ -576,6 +576,7 @@ nsSVGCairoPathGeometry::Flatten(nsSVGPathData **aData)
 {
   cairo_t *ctx = cairo_create(gSVGCairoDummySurface);
   GeneratePath(ctx, nsnull);
+  cairo_identity_matrix(ctx);
 
   *aData = new nsSVGPathData;
 
