@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=2 sw=2 et tw=78: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -76,10 +77,14 @@
 #include "nsUnicharUtils.h"
 #include "nsCSSFrameConstructor.h"
 #include "nsIBoxLayout.h"
+#include "nsIEventQueueService.h"
+#include "nsIServiceManager.h"
 #ifdef XP_WIN
 #include "nsISound.h"
 #endif
 
+static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
+
 const PRInt32 kMaxZ = 0x7fffffff; //XXX: Shouldn't there be a define somewhere for MaxInt for PRInt32
 
 
@@ -145,7 +150,6 @@ nsMenuPopupFrame::Release(void)
 //
 NS_INTERFACE_MAP_BEGIN(nsMenuPopupFrame)
   NS_INTERFACE_MAP_ENTRY(nsIMenuParent)
-  NS_INTERFACE_MAP_ENTRY(nsITimerCallback)
 NS_INTERFACE_MAP_END_INHERITING(nsBoxFrame)
 
 
@@ -169,6 +173,11 @@ nsMenuPopupFrame::Init(nsPresContext*  aPresContext,
 {
   nsresult rv = nsBoxFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
 
+  // Set up a mediator which can be used for callbacks on this frame.
+  mTimerMediator = new nsMenuPopupTimerMediator(this);
+  if (NS_UNLIKELY(!mTimerMediator))
+    return NS_ERROR_OUT_OF_MEMORY;
+
   // lookup if we're allowed to overlap the OS bar (menubar/taskbar) from the
   // look&feel object
   PRBool tempBool;
@@ -212,11 +221,27 @@ nsMenuPopupFrame::Init(nsPresContext*  aPresContext,
   widgetData.mBorderStyle = eBorderStyle_default;
   widgetData.clipSiblings = PR_TRUE;
 
+  PRBool isCanvas;
+  const nsStyleBackground* bg;
+  PRBool hasBG =
+    nsCSSRendering::FindBackground(aPresContext, this, &bg, &isCanvas);
+  PRBool viewHasTransparentContent = hasBG &&
+    (bg->mBackgroundFlags & NS_STYLE_BG_COLOR_TRANSPARENT) &&
+    !GetStyleDisplay()->mAppearance;
+  if (viewHasTransparentContent) {
+    nsCOMPtr<nsISupports> cont = GetPresContext()->GetContainer();
+    nsCOMPtr<nsIDocShellTreeItem> dsti = do_QueryInterface(cont);
+    PRInt32 type = -1;
+    if (!dsti || NS_FAILED(dsti->GetItemType(&type)) ||
+        type != nsIDocShellTreeItem::typeChrome)
+      viewHasTransparentContent = PR_FALSE;
+  }
+
   nsIContent* parentContent = aContent->GetParent();
   nsIAtom *tag = nsnull;
   if (parentContent)
     tag = parentContent->Tag();
-  widgetData.mDropShadow = !(tag && tag == nsXULAtoms::menulist);
+  widgetData.mDropShadow = !(viewHasTransparentContent || tag == nsXULAtoms::menulist);
   
   // XXX make sure we are hidden (shouldn't this be done automatically?)
   viewManager->SetViewVisibility(ourView, nsViewVisibility_kHide);
@@ -229,6 +254,7 @@ nsMenuPopupFrame::Init(nsPresContext*  aPresContext,
   ourView->CreateWidget(kCChildCID, &widgetData, nsnull, PR_TRUE, PR_TRUE);
 #endif   
 
+  ourView->GetWidget()->SetWindowTranslucency(viewHasTransparentContent);
   MoveToAttributePosition();
 
   return rv;
@@ -480,10 +506,11 @@ nsMenuPopupFrame::AdjustClientXYForNestedDocuments ( nsIDOMXULDocument* inPopupD
   // nsXULPopupListener).
 
   nsCOMPtr<nsIDOMNode> targetNode;
+  nsCOMPtr<nsIDOMXULDocument2> doc = do_QueryInterface(inPopupDoc);
   if (mContent->Tag() == nsXULAtoms::tooltip)
-    inPopupDoc->GetTooltipNode(getter_AddRefs(targetNode));
+    doc->TrustedGetTooltipNode(getter_AddRefs(targetNode));
   else
-    inPopupDoc->GetPopupNode(getter_AddRefs(targetNode));
+    doc->TrustedGetPopupNode(getter_AddRefs(targetNode));
 
   //NS_WARN_IF_FALSE(targetNode, "no popup/tooltip node on document!");
   nsCOMPtr<nsIContent> targetAsContent ( do_QueryInterface(targetNode) );
@@ -731,7 +758,37 @@ nsMenuPopupFrame::MovePopupToOtherSideOfParent ( PRBool inFlushAboveBelow, PRInt
 
 } // MovePopupToOtherSideOfParent
 
+struct nsASyncMenuActivation : public PLEvent
+{
+  nsASyncMenuActivation(nsIContent* aContent)
+    : mContent(aContent)
+  {
+  }
 
+  void HandleEvent() {
+    nsAutoString shouldDisplay, menuActive;
+    mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::menuactive, menuActive);
+    if (!menuActive.EqualsLiteral("true")) {
+      mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::menutobedisplayed,
+                        shouldDisplay);
+      if(shouldDisplay.EqualsLiteral("true")) {
+        mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::menuactive,
+                          NS_LITERAL_STRING("true"), PR_TRUE);
+      }
+    }
+  }
+
+  nsCOMPtr<nsIContent> mContent;
+};
+
+static void PR_CALLBACK HandleASyncMenuActivation(nsASyncMenuActivation* aEvent)
+{
+  aEvent->HandleEvent();
+}
+static void PR_CALLBACK DestroyASyncMenuActivation(nsASyncMenuActivation* aEvent)
+{
+  delete aEvent;
+}
 
 nsresult 
 nsMenuPopupFrame::SyncViewWithFrame(nsPresContext* aPresContext,
@@ -1114,8 +1171,25 @@ nsMenuPopupFrame::SyncViewWithFrame(nsPresContext* aPresContext,
   mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::menuactive, menuActive);
   if (!menuActive.EqualsLiteral("true")) {
     mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::menutobedisplayed, shouldDisplay);
-    if ( shouldDisplay.EqualsLiteral("true") )
-      mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::menuactive, NS_LITERAL_STRING("true"), PR_TRUE);
+    if ( shouldDisplay.EqualsLiteral("true") ) {
+      nsCOMPtr<nsIEventQueueService> eventService =
+        do_GetService(kEventQueueServiceCID);
+      NS_ENSURE_TRUE(eventService, NS_ERROR_FAILURE);
+      nsCOMPtr<nsIEventQueue> eventQueue;
+      eventService->GetThreadEventQueue(PR_GetCurrentThread(),
+                                        getter_AddRefs(eventQueue));
+      if (eventQueue) {
+        nsASyncMenuActivation* activation = new nsASyncMenuActivation(mContent);
+        if (activation) {
+          PL_InitEvent(activation, nsnull,
+                       (PLHandleEventProc) ::HandleASyncMenuActivation,
+                       (PLDestroyEventProc) ::DestroyASyncMenuActivation);
+          if (NS_FAILED(eventQueue->PostEvent(activation))) {
+            PL_DestroyEvent(activation);
+          }
+        }
+      }
+    }
   }
 
   return NS_OK;
@@ -1382,7 +1456,7 @@ NS_IMETHODIMP nsMenuPopupFrame::SetCurrentMenuItem(nsIMenuFrame* aMenuItem)
       mCloseTimer = do_CreateInstance("@mozilla.org/timer;1");
       nsCOMPtr<nsITimerInternal> ti = do_QueryInterface(mCloseTimer);
       ti->SetIdle(PR_FALSE);
-      mCloseTimer->InitWithCallback(this, menuDelay, nsITimer::TYPE_ONE_SHOT); 
+      mCloseTimer->InitWithCallback(mTimerMediator, menuDelay, nsITimer::TYPE_ONE_SHOT);
       mTimerMenu = mCurrentMenu;
     }
   }
@@ -1637,6 +1711,11 @@ nsMenuPopupFrame::FindMenuWithShortcut(nsIDOMKeyEvent* aKeyEvent, PRBool& doActi
 NS_IMETHODIMP 
 nsMenuPopupFrame::ShortcutNavigation(nsIDOMKeyEvent* aKeyEvent, PRBool& aHandledFlag)
 {
+  // See if we have a context menu open.
+  nsIMenuParent *contextMenu = GetContextMenu();
+  if (contextMenu)
+    return contextMenu->ShortcutNavigation(aKeyEvent, aHandledFlag);
+
   if (mCurrentMenu) {
     PRBool isOpen = PR_FALSE;
     mCurrentMenu->MenuIsOpen(isOpen);
@@ -1652,10 +1731,14 @@ nsMenuPopupFrame::ShortcutNavigation(nsIDOMKeyEvent* aKeyEvent, PRBool& aHandled
   nsIMenuFrame* result = FindMenuWithShortcut(aKeyEvent, action);
   if (result) {
     // We got one!
+    nsIFrame* frame = nsnull;
+    CallQueryInterface(result, &frame);
+    nsWeakFrame weakResult(frame);
     aHandledFlag = PR_TRUE;
     SetCurrentMenuItem(result);
-    if (action)
+    if (action && weakResult.IsAlive()) {
       result->Enter();
+    }
   }
 
   return NS_OK;
@@ -1691,6 +1774,7 @@ nsMenuPopupFrame::KeyboardNavigation(PRUint32 aKeyCode, PRBool& aHandledFlag)
   PRBool isContainer = PR_FALSE;
   PRBool isOpen = PR_FALSE;
   PRBool isDisabled = PR_FALSE;
+  nsWeakFrame weakFrame(this);
   if (mCurrentMenu) {
     mCurrentMenu->MenuIsContainer(isContainer);
     mCurrentMenu->MenuIsOpen(isOpen);
@@ -1699,13 +1783,19 @@ nsMenuPopupFrame::KeyboardNavigation(PRUint32 aKeyCode, PRBool& aHandledFlag)
     if (isOpen) {
       // Give our child a shot.
       mCurrentMenu->KeyboardNavigation(aKeyCode, aHandledFlag);
+      NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
     }
     else if (theDirection == eNavigationDirection_End &&
              isContainer && !isDisabled) {
       // The menu is not yet open. Open it and select the first item.
       aHandledFlag = PR_TRUE;
+      nsIFrame* frame = nsnull;
+      CallQueryInterface(mCurrentMenu, &frame);
+      nsWeakFrame weakCurrentFrame(frame);
       mCurrentMenu->OpenMenu(PR_TRUE);
+      NS_ENSURE_TRUE(weakCurrentFrame.IsAlive(), NS_OK);
       mCurrentMenu->SelectFirstItem();
+      NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
     }
   }
 
@@ -1736,6 +1826,7 @@ nsMenuPopupFrame::KeyboardNavigation(PRUint32 aKeyCode, PRBool& aHandledFlag)
     if (theDirection == eNavigationDirection_Start) {
       // Close it up.
       mCurrentMenu->OpenMenu(PR_FALSE);
+      NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
       // SelectMenu() so DOMMenuItemActive is fired for accessibility
       mCurrentMenu->SelectMenu(PR_TRUE);
       aHandledFlag = PR_TRUE;
@@ -1773,6 +1864,7 @@ nsMenuPopupFrame::HideChain()
   
   nsIFrame* frame = GetParent();
   if (frame) {
+    nsWeakFrame weakMenu(frame);
     nsIMenuFrame* menuFrame;
     if (NS_FAILED(CallQueryInterface(frame, &menuFrame))) {
       nsIPopupSetFrame* popupSetFrame = GetPopupSetFrame(mPresContext);
@@ -1783,7 +1875,9 @@ nsMenuPopupFrame::HideChain()
     }
    
     menuFrame->ActivateMenu(PR_FALSE);
+    NS_ENSURE_TRUE(weakMenu.IsAlive(), NS_OK);
     menuFrame->SelectMenu(PR_FALSE);
+    NS_ENSURE_TRUE(weakMenu.IsAlive(), NS_OK);
 
     // Get the parent.
     nsIMenuParent *menuParent = menuFrame->GetMenuParent();
@@ -1902,10 +1996,19 @@ PRBool
 nsMenuPopupFrame::IsValidItem(nsIContent* aContent)
 {
   nsIAtom *tag = aContent->Tag();
+  
+  PRBool skipNavigatingDisabledMenuItem;
+  GetPresContext()->LookAndFeel()->
+    GetMetric(nsILookAndFeel::eMetric_SkipNavigatingDisabledMenuItem,
+              skipNavigatingDisabledMenuItem);
 
-  return (tag == nsXULAtoms::menu ||
-          tag == nsXULAtoms::menuitem ||
-          tag == nsHTMLAtoms::option);
+  PRBool result = (tag == nsXULAtoms::menu ||
+                   tag == nsXULAtoms::menuitem ||
+                   tag == nsHTMLAtoms::option);
+  if (skipNavigatingDisabledMenuItem)
+    result = result && !IsDisabled(aContent);
+
+  return result;
 }
 
 PRBool 
@@ -1948,8 +2051,9 @@ nsMenuPopupFrame::MoveToAttributePosition()
   xPos = left.ToInteger(&err1);
   yPos = top.ToInteger(&err2);
 
-  if (NS_SUCCEEDED(err1) && NS_SUCCEEDED(err2))
-    MoveTo(xPos, yPos);
+  if (NS_SUCCEEDED(err1) && NS_SUCCEEDED(err2)) {
+    MoveToInternal(xPos, yPos);
+  }
 }
 
 
@@ -1964,6 +2068,13 @@ nsMenuPopupFrame::HandleEvent(nsPresContext* aPresContext,
 NS_IMETHODIMP
 nsMenuPopupFrame::Destroy(nsPresContext* aPresContext)
 {
+  // Null out the pointer to this frame in the mediator wrapper so that it 
+  // doesn't try to interact with a deallocated frame.
+  mTimerMediator->ClearFrame();
+
+  if (mCloseTimer)
+    mCloseTimer->Cancel();
+
   RemoveKeyboardNavigator();
   return nsBoxFrame::Destroy(aPresContext);
 }
@@ -2019,7 +2130,7 @@ nsMenuPopupFrame::GetFrameForPoint(const nsPoint& aPoint,
 //
 // The code below melds the two cases together.
 //
-NS_IMETHODIMP
+nsresult
 nsMenuPopupFrame::Notify(nsITimer* aTimer)
 {
   // Our timer has fired. 
@@ -2099,9 +2210,22 @@ nsMenuPopupFrame::MoveTo(PRInt32 aLeft, PRInt32 aTop)
   left.AppendInt(aLeft);
   top.AppendInt(aTop);
 
+  nsWeakFrame weakFrame(this);
   mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::left, left, PR_FALSE);
+  if (!weakFrame.IsAlive()) {
+    return;
+  }
   mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::top, top, PR_FALSE);
+  if (!weakFrame.IsAlive()) {
+    return;
+  }
 
+  MoveToInternal(aLeft, aTop);
+}
+
+void
+nsMenuPopupFrame::MoveToInternal(PRInt32 aLeft, PRInt32 aTop)
+{
   nsIView* view = GetView();
   NS_ASSERTION(view->GetParent(), "Must have parent!");
   
@@ -2136,3 +2260,42 @@ nsMenuPopupFrame::EnableRollup(PRBool aShouldRollup)
     CreateDismissalListener();
 }
 
+
+// nsMenuPopupTimerMediator implementation.
+NS_IMPL_ISUPPORTS1(nsMenuPopupTimerMediator, nsITimerCallback)
+
+/**
+ * Constructs a wrapper around an nsMenuFrame.
+ * @param aFrame nsMenuFrame to create a wrapper around.
+ */
+nsMenuPopupTimerMediator::nsMenuPopupTimerMediator(nsMenuPopupFrame *aFrame) :
+  mFrame(aFrame)
+{
+  NS_ASSERTION(mFrame, "Must have frame");
+}
+
+nsMenuPopupTimerMediator::~nsMenuPopupTimerMediator()
+{
+}
+
+/**
+ * Delegates the notification to the contained frame if it has not been destroyed.
+ * @param aTimer Timer which initiated the callback.
+ * @return NS_ERROR_FAILURE if the frame has been destroyed.
+ */
+NS_IMETHODIMP nsMenuPopupTimerMediator::Notify(nsITimer* aTimer)
+{
+  if (!mFrame)
+    return NS_ERROR_FAILURE;
+
+  return mFrame->Notify(aTimer);
+}
+
+/**
+ * Clear the pointer to the contained nsMenuFrame. This should be called
+ * when the contained nsMenuFrame is destroyed.
+ */
+void nsMenuPopupTimerMediator::ClearFrame()
+{
+  mFrame = nsnull;
+}
