@@ -479,7 +479,7 @@ nsParseMailMessageState::nsParseMailMessageState()
 
   // setup handling of custom db headers, headers that are added to .msf files
   // as properties of the nsMsgHdr objects, controlled by the 
-  // pref mailnews.customDBHeaders.
+  // pref mailnews.customDBHeaders, a space-delimited list of headers.
   // E.g., if mailnews.customDBHeaders is "X-Spam-Score", and we're parsing
   // a mail message with the X-Spam-Score header, we'll set the 
   // "x-spam-score" property of nsMsgHdr to the value of the header.
@@ -490,7 +490,7 @@ nsParseMailMessageState::nsParseMailMessageState()
   {
      pPrefBranch->GetCharPref("mailnews.customDBHeaders",  getter_Copies(customDBHeaders));
      ToLowerCase(customDBHeaders);
-     m_customDBHeaders.ParseString(customDBHeaders, ", ");
+     m_customDBHeaders.ParseString(customDBHeaders, " ");
      if (m_customDBHeaders.Count())
      {
        m_customDBHeaderValues = new struct message_header [m_customDBHeaders.Count()];
@@ -1684,6 +1684,7 @@ PRInt32 nsParseNewMailState::PublishMsgHeader(nsIMsgWindow *msgWindow)
           m_newMsgHdr->OrFlags(MSG_FLAG_NEW, &newFlags);
         
         m_mailDB->AddNewHdrToDB(m_newMsgHdr, PR_TRUE);
+        NotifyGlobalListeners(m_newMsgHdr);
       }
     } // if it was moved by imap filter, m_parseMsgState->m_newMsgHdr == nsnull
     m_newMsgHdr = nsnull;
@@ -1691,6 +1692,14 @@ PRInt32 nsParseNewMailState::PublishMsgHeader(nsIMsgWindow *msgWindow)
   return 0;
 }
 
+void nsParseNewMailState::NotifyGlobalListeners(nsIMsgDBHdr *newHdr)
+{
+  if (!m_notificationService)
+    m_notificationService = do_GetService("@mozilla.org/messenger/msgnotificationservice;1");
+  if (m_notificationService)
+    m_notificationService->NotifyItemAdded(newHdr);
+}
+
 nsresult nsParseNewMailState::GetTrashFolder(nsIMsgFolder **pTrashFolder)
 {
   nsresult rv=NS_ERROR_UNEXPECTED;
@@ -2017,7 +2026,8 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
   {
     PRInt32 numNewMessages;
     m_downloadFolder->GetNumNewMessages(PR_FALSE, &numNewMessages);
-    m_downloadFolder->SetNumNewMessages(numNewMessages - 1);
+    if (numNewMessages > 0)
+      m_downloadFolder->SetNumNewMessages(numNewMessages - 1);
   }
   return rv;
 }
@@ -2293,6 +2303,7 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
         }
       }
       destMailDB->AddNewHdrToDB(newHdr, PR_TRUE);
+      NotifyGlobalListeners(newHdr);
       m_msgToForwardOrReply = newHdr;
     }
   }
