@@ -72,10 +72,23 @@ pref("mailnews.headers.showUserAgent",       false);
 // is displayed in the message pane or not...
 pref("mailnews.headers.showOrganization",    false);
 
+// space-delimited list of extra headers to show in msg header display area.
+pref("mailnews.headers.extraExpandedHeaders", "");
+
 // If true, delete will use the direction of the sort order
 // in determining the next message to select. 
 pref("mail.delete_matches_sort_order", false);
 
+// default sort order settings (when creating new folder views)
+// sort_order is an int value reflecting nsMsgViewSortOrder values
+//   as defined in nsIMsgDBView.idl (ascending = 1, descending = 2)
+// sort_type is an int value reflecting nsMsgViewSortType values
+//   as defined in nsIMsgDBView.idl (byDate = 18, byThread = 22 etc.)
+pref("mailnews.default_sort_order", 1); // for Mail/RSS/... (nsMsgDatabase)
+pref("mailnews.default_sort_type", 18); //
+pref("mailnews.default_news_sort_order", 1); // for News (nsNewsDatabase)
+pref("mailnews.default_news_sort_type", 22); //
+
 // mailnews tcp read+write timeout in seconds.
 pref("mailnews.tcptimeout", 60);
 
@@ -332,8 +345,8 @@ pref("ldap_2.servers.default.attrmap.HomeZipCode", "mozillaHomePostalCode");
 pref("ldap_2.servers.default.attrmap.WorkCountry", "c,countryname");
 pref("ldap_2.servers.default.attrmap.HomeCountry", "mozillaHomeCountryName");
 pref("ldap_2.servers.default.attrmap.JobTitle", "title");
-pref("ldap_2.servers.default.attrmap.Department", "department,departmentnumber,ou,orgunit");
-pref("ldap_2.servers.default.attrmap.Company", "company,o");
+pref("ldap_2.servers.default.attrmap.Department", "ou,department,departmentnumber,orgunit");
+pref("ldap_2.servers.default.attrmap.Company", "o,company");
 pref("ldap_2.servers.default.attrmap._AimScreenName", "nsAIMid,nscpaimscreenname");
 pref("ldap_2.servers.default.attrmap.WebPage1", "mozillaWorkUrl,workurl");
 pref("ldap_2.servers.default.attrmap.WebPage2", "mozillaHomeUrl,homeurl");
@@ -372,6 +385,7 @@ pref("ldap_2.prefs_migrated",      false);
 
 pref("mailnews.confirm.moveFoldersToTrash", true);
 
+// space-delimited list of extra headers to add to .msf file
 pref("mailnews.customDBHeaders", "");
 
 pref("mailnews.reuse_message_window", true);
@@ -582,6 +596,10 @@ pref("mail.trusteddomains", "");
 
 pref("mail.imap.use_status_for_biff", true);
 
+pref("mail.quota.mainwindow_threshold.show", 75); // in percent. when the quota meter starts showing up at all. decrease this for it to be more than a warning.
+pref("mail.quota.mainwindow_threshold.warning", 80); // when it gets yellow
+pref("mail.quota.mainwindow_threshold.critical", 95); // when it gets red
+
 // Pref controlling confirmation of folder deletion on empty trash
 pref("mail.imap.confirm_emptyTrashFolderDeletion", false);
 // Pref controlling the updates on the pre-configured accounts.
